VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCommands"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Contains all accessible log on commands used in clsTelnet (ExecuteLogon)

' Different modules
Public Session As clsSession
Public Terminal As clsTerminal
Public Telnet As clsTelnet

' Saved variables
Public SavedUsername As String
Public SavedPassword As String
Public SavedAttribute As String
Public SavedLevel As Long
Public SavedCommandLine As String
Public SavedLine As String

' Used to access an objects properties
Private TLIApp As New TLIApplication

Private Sub SetPrivilegies(User As clsUser, Flags As String, Enable As Boolean)

    Dim aAttributes As Variant, Tell As Long, bEnableAll As Boolean
    
    ' Set the different attributes we can change
    aAttributes = Array("logon", "logoff", "shell", "setcmd", "list", "setpriv", _
                         "adduser", "deluser", "shutdown", "send")
                         
    ' See if we can enable all
    bEnableAll = (InStr(1, Flags, "*") <> 0)

    ' Go through the list, ...
    For Tell = LBound(aAttributes) To UBound(aAttributes)
        
        ' Enabling or disabling each attribute
        CallByName User, "Allow" & aAttributes(Tell), VbLet, _
         IIf(CBool(InStr(1, Flags, aAttributes(Tell), vbTextCompare) <> 0) Or bEnableAll, Enable, Not Enable)
    
    Next

End Sub

' Retrieves a user using a name (for modification). Considers all restrictions, except user password.
Private Function OpenUser(UserName As String, Optional Sender As String, Optional IgnoreSelf As Boolean) As clsUser

    ' Firstly, see if the user exits
    Set OpenUser = Terminal.FindUser(UserName)

    ' Continue if we found the user
    If Not (OpenUser Is Nothing) Then
    
        ' See if the users security level is lower than this user (or we have the same user)
        If OpenUser.SecurityLevel < Session.CurrentUser.SecurityLevel Or _
         (Session.CurrentUser.UserName = UserName) Then
    
            ' Then see if we're supposed to ignore the current user
            If IgnoreSelf And (UserName = Session.CurrentUser.UserName) Then
            
                ' This action cannot be perfomed on one self
                Telnet.SendLine Respond_CannotBePerfomedOnSelf, , Sender
            
            Else
    
                ' All is OK.
                Exit Function
    
            End If
    
        Else
            ' We are not allowed to modify this user.
            Telnet.SendLine Respond_SecurityLevelToHigh, , Sender
        End If
    
    Else
        ' Inform about this
        Telnet.SendLine Respond_UserNotFound, , Sender
    End If
    
    ' Return nothing otherwise
    Set OpenUser = Nothing

End Function

Public Sub Echo(Line As String)

    ' Return the paramenter
    Session.SendData Line & vbCrLf

End Sub

Public Sub User(Value As String)

    ' Save the username
    SavedUsername = Value

End Sub

Public Sub Pass(Value As String)

    ' Save the password
    SavedPassword = Value

End Sub

Public Sub Attr(Value As String)

    ' Save the attribute name
    SavedAttribute = Value

End Sub

Public Sub level(Value As String)

    ' Save the security level to use
    SavedLevel = Val(Value)

End Sub

Public Sub SetPass(newPassword As String)

    ' Set the current user's password
    Session.CurrentUser.UserPassword = newPassword

End Sub

Public Sub Ping(Optional sParam As String)

    ' Just used to make sure the server is alive.
    Telnet.SendLine Respond_Pong

End Sub

Public Sub AddUser(Flags As String)

    Dim oUser As clsUser

    ' Make sure the username dosen't contain space, ...
    If InStr(1, SavedUsername, " ") <> 0 Then
        Telnet.SendLine Respond_CannotContainSpace, , "Cannot add user.", "Username"
        Exit Sub
    ElseIf IsNumeric(SavedUsername) Then
        ' ... or only contain numbers.
        Telnet.SendLine Respond_CannotBeNumeric, , "Cannot add user.", "Username"
        Exit Sub
    End If

    ' See if this procedure is enabled
    If Session.CurrentUser.AllowAddUser Then

        ' Make sure the user dosen't already exist
        If Terminal.FindUser(SavedUsername) Is Nothing Then
        
            ' This users security level cannot be higher than the current
            If SavedLevel <= Session.CurrentUser.SecurityLevel Then
            
                ' Create this user with the given password
                Set oUser = Terminal.CreateUser(SavedUsername, SavedPassword)
            
                ' Set the privilegies
                SetPrivilegies oUser, Flags, True
                
                ' Set the security level
                oUser.SecurityLevel = SavedLevel
                
                ' Inform about the success
                Telnet.SendLine Respond_UserCreated
            
            Else
                Telnet.SendLine Respond_SecurityLevelToHigh, , "Cannot add user."
            End If
        
        Else
            ' The user is already present.
            Telnet.SendLine Respond_UserAlreadyExist, , "Cannot add user."
        End If
        
    Else
        Telnet.SendLine Respond_AccessDenied
    End If

End Sub

Public Sub DelUser(UserName As String)

    Dim oUser As clsUser

    ' Make sure we're allowed to run this
    If Session.CurrentUser.AllowDelUser Then

        ' Attempt to open the user
        Set oUser = OpenUser(UserName, "Cannot delete.")
        
        ' If we succeeded, continue
        If Not (oUser Is Nothing) Then
        
            ' Kill the user
            If Terminal.KillUser(UserName) Then
                ' Inform about this
                Telnet.SendLine Respond_UserDeleted
            End If
        
        End If

    Else
        Telnet.SendLine Respond_AccessDenied
    End If
    
End Sub

Public Sub SetAttr(Value As String)

    Dim oUser As clsUser, oMember As MemberInfo, sSender As String

    ' Set the sender variable
    sSender = "Cannot set attribute."

    ' See if this procedure is enabled
    If Session.CurrentUser.AllowSetAttr Then
        
        ' Retrieve the selected user
        Set oUser = OpenUser(SavedUsername, sSender)

        ' Make sure we could open the user
        If Not (oUser Is Nothing) Then

            ' Attempt to retrieve the property
            Set oMember = TLIApp.InterfaceInfoFromObject(oUser).GetMember(SavedAttribute)

            ' Make sure the property exist
            If Not (oMember Is Nothing) Then
            
                ' Set this attribute
                CallByName oUser, SavedAttribute, VbLet, Value
            
                ' Inform about the success
                Telnet.SendLine Respond_AttributeSet

            Else
                Telnet.SendLine Respond_AttributeDosentExist, , sSender
            End If
            
        End If
        
    Else
        Telnet.SendLine Respond_AccessDenied
    End If

End Sub

Public Sub GetAttr(Optional sParam As String)

    Dim oUser As clsUser, oMember As MemberInfo, sSender As String

    ' Set the sender variable
    sSender = "Cannot get attribute."

    ' See if this procedure is enabled
    If Session.CurrentUser.AllowSetAttr Then
        
        ' Retrieve the selected user (do not allow security level to be changed on the current user)
        Set oUser = OpenUser(SavedUsername, sSender, (LCase(SavedAttribute) = "securitylevel"))

        ' Make sure we could open the user
        If Not (oUser Is Nothing) Then

            ' Attempt to retrieve the property
            Set oMember = TLIApp.InterfaceInfoFromObject(oUser).GetMember(SavedAttribute)

            ' Make sure the property exist
            If Not (oMember Is Nothing) Then
            
                ' Return the value
                Telnet.SendLine Respond_AttributeGet, , CallByName(oUser, SavedAttribute, VbGet)
            
            Else
                Telnet.SendLine Respond_AttributeDosentExist, , sSender
            End If
            
        End If
        
    Else
        Telnet.SendLine Respond_AccessDenied
    End If

End Sub

Public Sub Allow(Flags As String, Optional Enable As Boolean = True)

    Dim oUser As clsUser

    ' See if this procedure is enabled
    If Session.CurrentUser.AllowSetPriv Then
        
        ' Retrieve the selected user
        Set oUser = OpenUser(SavedUsername, "Cannot set privilegies.")

        ' Make sure we could open the user
        If Not (oUser Is Nothing) Then

            ' Set the privilegies
            SetPrivilegies oUser, Flags, Enable
            
            ' Inform about the success
            Telnet.SendLine Respond_PrivilegiesSet
            
        End If
        
    Else
        Telnet.SendLine Respond_AccessDenied
    End If
    
End Sub

Public Sub Disallow(Flags As String)

    ' Do just the opposite of the allow function.
    Allow Flags, False
    
End Sub

Public Sub Send(Message As String)

    Dim oUser As clsUser, oSession As clsSession

    ' See if this procedure is enabled
    If Session.CurrentUser.AllowSend Then
        
        ' Retrieve the selected user
        Set oUser = Terminal.FindUser(SavedUsername)
        
        ' Make sure the user already exist
        If Not (oUser Is Nothing) Then

            ' Send the message
            For Each oSession In Terminal.Sessions
                ' Send the message if the name correspond or the user name set is *.
                If (oSession.CurrentUser.UserName = SavedUsername) Or SavedUsername = "*" Then
                    
                    ' Do not send the message to our self
                    If Not (oSession Is Session) Then
                    
                        ' Send this mesage
                        oSession.SendData Message & vbCrLf
                        
                    End If
                    
                End If
            Next
            
            ' Inform about the success
            Telnet.SendLine Respond_LineSent
        
        Else
            ' The username is invalid.
            Telnet.SendLine Respond_UserNotFound, , "Cannot send data"
        End If
        
    Else
        Telnet.SendLine Respond_AccessDenied
    End If

End Sub

Public Sub Shutdown(Optional sParam As String)

    ' See if this procedure is enabled
    If Session.CurrentUser.AllowShutdown Then
        
        ' Ask the terminal to shutdown
        Terminal.RequestShutdown Session.CurrentUser
        
    Else
        Telnet.SendLine Respond_AccessDenied
    End If

End Sub

Public Sub Logon(Optional sParam As String)

    ' Log on with the saved username and password
    Connect SavedUsername & " " & SavedPassword

End Sub

Public Sub Connect(sParam As String)

    Dim oUser As clsUser, aVar As Variant

    ' Split the paramenter into two
    aVar = Split(sParam, " ", 2)

    ' Make sure the user has the privileges
    If Session.CurrentUser.AllowLogon Then

        ' Attemt to find the user
        Set oUser = Terminal.FindUser(CStr(aVar(0)))
        
        ' See if the password is correct
        Select Case Session.Logon(oUser, CStr(aVar(1)))
            Case 0 ' Success
                Telnet.SendLine Respond_LoggedOn
            Case -1
                Telnet.SendLine Respond_UserExpired, , "Cannot logon."
            Case -2
                Telnet.SendLine Respond_IncorrectPassword, , "Cannot logon."
            Case -3
                Telnet.SendLine Respond_SessionsExceeded, , "Cannot logon."
            Case -4
                Telnet.SendLine Respond_UserNotFound, , "Cannot logon."
            Case -5
                ' This is caused because the application denied the logon. Most likely due to many logon attempts.
                Telnet.SendLine Respond_LogonDenied, , "Cannot logon."
        End Select
    
    Else
        ' Inform that this command is cut off
        Telnet.SendLine Respond_AccessDenied
    End If

End Sub

Public Sub LogOff(Optional sParam As String)

    ' See if the command is allowed
    If Session.CurrentUser.AllowLogoff Then
    
        ' Use the default user from now
        Session.Logon Terminal.DefaultUser
    
        ' Inform that we're now logged of
        Telnet.SendLine Respond_LoggedOff
    
    Else
        
        ' Inform that this command is cut off
        Telnet.SendLine Respond_AccessDenied
        
    End If

End Sub

Public Sub SetCmd(CommandLine As String)

    ' See if the command is allowed
    If Session.CurrentUser.AllowSetCmd Then
    
        ' Set the command line to use
        SavedCommandLine = CommandLine
    
        ' Respond that the command line was set
        Telnet.SendLine Respond_CommandLineSet
    
    Else
        ' Inform that this command is cut off
        Telnet.SendLine Respond_AccessDenied
    End If

End Sub

Public Sub Shell(AppPath As String)

    Dim bCancel As Boolean

    ' See if we are allowed
    If Session.CurrentUser.AllowShell Then
        
        ' Clear the buffer to avoid errors
        Session.DataBuffer = ""
        
        ' Inform the parent that we're about to execute an application
        bCancel = Session.Parent.InvokeEvent(2, SavedCommandLine, AppPath)
        
        ' See if we should execute
        If Not bCancel Then
        
            ' Execute an application
            If Not Telnet.Shell.Execute(SavedCommandLine, Session.CurrentUser.HomeFolder & AppPath) Then
                ' Inform if the execute failed.
                Telnet.SendLine Respond_ShellFailed
            Else
                Telnet.SendLine Respond_ShellFinished
            End If
        
        Else
        
            ' If not, inform the user.
            Telnet.SendLine Respond_ShellAborted
        
        End If
        
    Else
        ' Inform that this command is cut off
        Telnet.SendLine Respond_AccessDenied
    End If

End Sub

Public Sub List(All As String)

    Dim oSession As clsSession, oUser As clsUser, sReturn As String
    
    ' See if the command is allowed
    If Session.CurrentUser.AllowList Then
    
        ' See if the paramenter is set, ...
        If LenB(All) <> 0 Then
        
            ' If so, go through all users, ...
            For Each oUser In Terminal.Users
                ' ... adding its username to the list.
                If oUser.UserName <> "Default" Then
                    sReturn = sReturn & oUser.UserName & vbCrLf
                End If
            Next
        
        Else
        
            ' If not, go through all users logged on, ...
            For Each oSession In Terminal.Sessions
                ' ... adding its username to the list.
                If oSession.CurrentUser.UserName <> "Default" Then
                    sReturn = sReturn & oSession.CurrentUser.UserName & vbCrLf
                End If
            Next
            
        End If
        
        ' Return a list of all users logged on
        Telnet.SendLine IIf(LenB(All) = 0, Respond_UsersLoggedOn, Respond_AllUsers), sReturn

    Else
        ' Inform that this command is cut off
        Telnet.SendLine Respond_AccessDenied
    End If
    
End Sub

