VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsSession"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Different session-related events
Public Event SessionClosed()
Public Event SessionConnected()
Public Event SessionDataRecieved(Data As Variant)
Public Event SessionDataSending(Data As Variant)
Public Event SessionError(ByVal Number As Integer, Description As String)
Public Event SessionUserLoggedIn()
Public Event SessionUserLoggedOff()
Public Event SessionExiting()

' The CSocket connection object
Public WithEvents Socket As clsSocket
Attribute Socket.VB_VarHelpID = -1

' The parent object
Public Parent As clsTerminal

' Object used to add more functionality to the object
Public EventTrapper As clsTelnet

' The delimiter to use (no delimiter mens this feature is disabled)
Public LineDelimiter As String

' Whether or not the connection is currently sending data
Public Sending As Boolean

' Used to log all data that is sent and recieved
Public Log As New clsLogging

' The current user
Public CurrentUser As clsUser

' The amount of unsuccessful logon attempts
Public LogonAttempts As Long

' The date this session was initialized
Public SessionStart As Date
Public SessionLast As Date

' Contains the amount of data send and recieved
Public BytesSent As Double
Public BytesRecieved As Double

' Buffer used to store the retrieved information
Public DataBuffer As String

' Used to save the remote host
Public RemoteHost As String

' Variables to store different options
Private oSupressGoAhead As Boolean

Public Sub ExitSession()

    ' Inform that we're now closing the session (but before the connection is lost)
    RaiseEvent SessionExiting

    ' Close the connection
    Socket_OnClose

End Sub

Public Property Get SessionIdle() As Date
    
    ' Returns the idle-time (inactive time)
    SessionIdle = Now - SessionLast

End Property

Public Sub InitializeSession(sFileName As String, oParent As clsTerminal, Optional oEventTrapper As Object)

    ' See if the log file is enabled
    If sFileName <> "" Then
        
        ' Initialize the logging
        Log.InitializeLog sFileName
    
    End If
    
    ' Then, set the parent and the event trapper
    Set Parent = oParent
    Set EventTrapper = oEventTrapper
    
    ' Reference this class and the parent (as a consequence, the trapper class MUST support these properties)
    Set EventTrapper.Session = Me
    Set EventTrapper.Terminal = Parent
        
    ' Set the date this session was created
    SessionStart = Now
    SessionLast = Now
    
    ' Basically, we're connected to the client by now
    RaiseEvent SessionConnected

End Sub

Public Sub SendData(vData As Variant, Optional Asyncronous As Boolean)

    Dim aData() As Byte

    ' See if we're supposed to send asyncronously
    If Not Asyncronous Then

        ' As this process is supposed to be syncronous, wait till transfer is ready
        Do While Sending
            DoEvents
            Sleep 1
        Loop
        
        ' Set the sending state
        Sending = True
    
    End If
    
    ' Inform that we're sendng this data
    RaiseEvent SessionDataSending(vData)

    ' Send the actual data
    Socket.SendData vData
    
    ' Let other events be processed
    DoEvents

    ' Inform about this sending
    If varType(vData) = vbString Then
        BytesSent = BytesSent + Len(vData)
        Log.AppendLine CStr(vData)
    Else
        BytesSent = BytesSent + UBound(vData) - LBound(vData) + 1
        Log.AppendLine StrConv(vData, vbUnicode)
    End If

End Sub

Private Sub Class_Terminate()

    ' Clear variables
    Set EventTrapper = Nothing

End Sub

Private Sub Socket_OnClose()

    ' Log off the current user
    LogOff

    ' Close this session
    Set Socket = Nothing
    
    ' Inform about this
    RaiseEvent SessionClosed
    
    ' Remove the session from the list
    Parent.KillSession Me

End Sub

Private Sub Socket_OnConnect()
    
    ' Clear send state
    Sending = False
    
    ' Inform that the connection has been established
    RaiseEvent SessionConnected
    
End Sub

Private Sub Socket_OnDataArrival(ByVal bytesTotal As Long)

    Dim sData As String

    ' Retrieve the incomming information
    Socket.GetData sData, vbByte + vbArray
    
    ' Process this data
    ProcessData sData

End Sub

Private Sub Class_Initialize()

    ' Initialize the socket
    Set Socket = New clsSocket
        
    ' Set the default attributes
    LineDelimiter = vbCrLf

End Sub

Private Sub ProcessData(Optional sData As String)

    Dim Tell As Long, iPos As Long, iLine As Long
    
    ' Retrieve the lenght of the delimter
    iLine = Len(LineDelimiter)

    ' See if we'll just proceed with the data, or only return when a line is complete
    If iLine > 0 Then
    
        ' Make sure there's any data to process
        If LenB(sData) > 0 Then
    
            ' Append the text. Since the internal string is UNICODE, and telnet is ASCII, we must convert it.
            DataBuffer = DataBuffer & StrConv(sData, vbUnicode)
            
        End If

        ' See if there's anything we can extract
        If LenB(DataBuffer) > 0 Then
    
            ' Attempt to find the position to the end of the line
            iPos = InStr(1, DataBuffer, LineDelimiter, vbTextCompare)
            
            ' If found, ...
            If iPos > 0 Then
            
                ' ... return that segment.
                DataRecieved Mid(DataBuffer, 1, iPos + iLine)

                ' Remove that portion of the buffer
                DataBuffer = Mid(DataBuffer, iPos + iLine)
            
                ' Continue processing the data
                ProcessData
            
            End If
        
        End If
    
    Else
        ' Inform that the data has arrived
        DataRecieved sData
    End If

End Sub

' Log on a given user
Public Function Logon(User As clsUser, Optional Password As String) As Long

    Dim bCancel As Boolean, oReturn As Variant

    ' Store the object in a variant
    Set oReturn = Me

    ' Inform about the logon request
    bCancel = Parent.InvokeEvent(1, oReturn, User)
    
    ' If the event was cancelled, ...
    If bCancel Then
        
        ' ... return the propper error code.
        Logon = -5
        
        ' ... do so.
        Exit Function
        
    End If

    ' Make sure the given user exists
    If Not (User Is Nothing) Then

        ' Make sure the user isn't expired
        If Now < User.Expire Then
    
            ' And make sure the user hasn't exceeded its maximum amount of sessions
            If User.Sessions < User.MaxSessions Or User.MaxSessions < 0 Then
        
                ' Make sure the password is correct
                If User.IsOK(Password) Then
        
                    ' Clear the amount of logon attempts
                    LogonAttempts = 0
        
                    ' Automatically log off
                    LogOff
        
                    ' Set this user as the current user
                    Set CurrentUser = User
                
                    ' Execute the logon text, if set
                    If LenB(User.OnLogon) <> 0 Then
                        DataRecieved User.OnLogon
                    End If
                    
                    ' Inform that the user has logged on
                    RaiseEvent SessionUserLoggedIn
                
                Else
                    
                    ' Increase the amount of logon attempts
                    LogonAttempts = LogonAttempts + 1
                
                    ' Invalid password.
                    Logon = -2
                    
                End If
            
            Else
                Logon = -3 ' Sessions exceeded
            End If
            
        Else
            Logon = -1 ' Expired user.
        End If
    Else
        Logon = -4 ' User not found
    End If

End Function

' Logs off the current user
Public Sub LogOff()

    ' Make sure there's a user logged on
    If Not (CurrentUser Is Nothing) Then

        ' Firstly, decrease the amount of sessions connected to this user
        CurrentUser.Sessions = CurrentUser.Sessions - 1
    
        ' Then, inform about this
        RaiseEvent SessionUserLoggedOff
        
        ' And set the user to nothing
        Set CurrentUser = Nothing
    
    End If

End Sub

Public Sub DataRecieved(Data As String, Optional IgnoreLogging As Boolean)

     ' Make sure there's any data to inform about
    If LenB(Data) > 0 Then
        
        ' Save that something was recieved
        SessionLast = Now
        
        ' Log this data arrival
        If Not ((Log Is Nothing) Or IgnoreLogging) Then
            Log.AppendLine Data
        End If
        
        ' Increase the amount of bytes recieved (as we really should measure ASCII characters, we'll use Len)
        BytesRecieved = BytesRecieved + Len(Data)
        
        ' Inform about this
        RaiseEvent SessionDataRecieved(Data)
        
    End If

End Sub

Private Sub Socket_OnError(ByVal Number As Integer, Description As String, ByVal Scode As Long, ByVal Source As String, ByVal HelpFile As String, ByVal HelpContext As Long, CancelDisplay As Boolean)

    ' Inform about the error
    RaiseEvent SessionError(Number, Description)
    
    ' Clear sending state
    Sending = False

End Sub

Private Sub Socket_OnSendComplete()

    ' All sending is completed
    Sending = False
    
End Sub


