VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsTelnet"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Used to add telnet-speficic operations to the modules (to allow the modules to be reused more efficiently)

' Contains all the "supported" commands
Public Enum Telnet_Commands
    Telnet_SE = 240
    Telnet_NOP = 241
    Telnet_DM = 242
    Telnet_BRK = 243
    Telnet_IP = 244
    Telnet_AO = 245
    Telnet_AYT = 246
    Telnet_EC = 247
    Telnet_EL = 248
    Telnet_GA = 249
    Telnet_SB = 250
    Telnet_WILL = 251
    Telnet_WONT = 252
    Telnet_DO = 253
    Telnet_DONT = 254
    Telnet_IAC = 255
End Enum

' Contains the different telnet options
Public Enum Telnet_Options
    Telnet_Echo = 1
    Telnet_SupressGoAhead = 3
    Telnet_Status = 5
    Telnet_TimingMark = 6
    Telnet_TerminalType = 24
    Telnet_WindowSize = 31
    Telnet_TerminalSpeed = 32
    Telnet_RemoteFlowControl = 33
    Telnet_Linemode = 34
    Telnet_EnvironmentVariables = 39
End Enum

Public Enum Telnet_Environ
    Environ_IS = 0
    Environ_Send = 1
    Environ_Info = 2
    Environ_Var = 0
    Environ_Value = 1
    Environ_ESC = 2
    Environ_USERVAR = 3
End Enum

' Different logon responses
Public Enum LogonResponses
    Respond_AccessDenied = 100
    Respond_UserNotFound = 101
    Respond_IncorrectPassword = 102
    Respond_CommandNotRecognized = 103
    Respond_ShellFailed = 104
    Respond_UserAlreadyExist = 105
    Respond_UserExpired = 106
    Respond_OutOfStackSpace = 107
    Respond_SessionsExceeded = 108
    Respond_SecurityLevelToHigh = 109
    Respond_AttributeDosentExist = 110
    Respond_CannotBePerfomedOnSelf = 111
    Respond_LogonDenied = 112
    Respond_ShellAborted = 113
    Respond_CannotContainSpace = 114
    Respond_CannotBeNumeric = 115
    Respond_ShellFinished = 202
    Respond_LoggedOn = 203
    Respond_LoggedOff = 204
    Respond_PrivilegiesSet = 205
    Respond_Pong = 206
    Respond_UserCreated = 207
    Respond_UserDeleted = 208
    Respond_CommandLineSet = 209
    Respond_LineSent = 210
    Respond_AttributeSet = 211
    Respond_UsersLoggedOn = 300
    Respond_AllUsers = 301
    Respond_SendingHelp = 302
    Respond_AttributeGet = 303
End Enum

' Executed when a command is parsed
Public Event CommandExecuting(Command As Telnet_Commands, Options As Telnet_Options, Paramenters() As Byte, Output As String, Cancel As Boolean)
Public Event LogonExecuting(CommandName As String, Paramenter As String, Cancel As Boolean)

' The session-object we're currently connected to
Public WithEvents Session As clsSession
Attribute Session.VB_VarHelpID = -1

' The terminal object
Public Terminal As clsTerminal

' The related shell-object
Public WithEvents Shell As clsShell
Attribute Shell.VB_VarHelpID = -1

' Whether or not to enable processing of commands
Public EnableCmd As Boolean

' Whether or not to automatically convert enviroment variables
Public EnableEnviroment As Boolean

' Different variables
Public SilienceShell As Boolean
Public SuppressGoAhead As Boolean ' Note, we don't support GA.
Public EnableEcho As Boolean
Public SavedLine As String ' The last processed line

' The current size of the client's window
Public WindowWidth As Long
Public WindowHeight As Long

' The string to send when the user asks for help
Private HelpText As String

' Class that contains the commands we use
Private Commands As New clsCommands

' The maximum amount of times exec (in ExecuteLogon) can call itself
Private Const MaxTree = 31

Private Sub Class_Initialize()

    ' Initialize the shell-object
    Set Shell = New clsShell
    
    ' Reference this class
    Set Commands.Telnet = Me
    
    ' Set default variables
    EnableCmd = True
    EnableEnviroment = True
    WindowWidth = 80
    WindowHeight = 24
    
    ' Initialize the string to send when the user request help
    HelpText = "USER [name]     Sets the username to use when logging on." & vbCrLf & _
               "PASS [text]     Sets the password to use when logging on." & vbCrLf & _
               "ATTR [name]     Sets the name of the attribute to get/set." & vbCrLf & _
               "LEVEL [number]  Sets the next created user's security level." & vbCrLf & _
               "ADDUSER [flags] Creates a user using USER and PASS and sets its privileges," & vbCrLf & _
               "                enabling when the name of the command is present. If * " & vbCrLf & _
               "                is present, all privilegies will be enabled." & vbCrLf & _
               "DELUSER [name]  Deletes a user by name." & vbCrLf & _
               "ALLOW [flags]   Enables a given set of privileges for a user." & vbCrLf & _
               "DISALLOW [flg.] Disables a given set of privileges." & vbCrLf & _
               "SHELL [app]     Executes the given command and routes input" & vbCrLf & _
               "                to the program and its output to the user." & vbCrLf & _
               "SETCMD [line]   Sets the command line to use when executing." & vbCrLf & _
               "ECHO [message]  Returns the given message." & vbCrLf & _
               "EXEC [code]     Executes the given code using these functions." & vbCrLf & _
               "SEND [line]     Sends a line to all users logged on with the" & vbCrLf & _
               "                name as set in USER. If USER is set to *, all users" & vbCrLf & _
               "                recieves the line. The sender is always ignored." & vbCrLf & _
               "SETATTR [value] Sets any users (USER) attribute (ATTR) to the value." & vbCrLf & _
               "GETATTR         Gets the attribute (PASS) of any user (ATTR)." & vbCrLf & _
               "SETPASS [pass]  Changes the current users password." & vbCrLf & _
               "SHUTDOWN        Terminate the server." & vbCrLf & _
               "LOGON           Logs on the terminal with the username and password set." & vbCrLf & _
               "LOGOFF          Logs off the current user." & vbCrLf
    HelpText = HelpText & _
               "CONNECT [u] [p] Similar to LOGON except it uses the paramenters to logon. " & vbCrLf & _
               "LIST [ALL]      Returns a list of all logged in users. If the paramenter" & vbCrLf & _
               "                is set, it will return a list of all users." & vbCrLf & _
               "HELP            Shows this text." & vbCrLf & _
               "PING            Is alive-command. Returns 'pong'." & vbCrLf & _
               "REM             Marks a line for comment." & vbCrLf & _
               "EXIT            Close the connection to the server." & vbCrLf
    ' Had to split the string into two due to a "line continuation limit".

End Sub

Private Sub Class_Terminate()

    ' WE *MUST* terminate the loop, otherwise the entire application will not exit.
    Shell.Running = False

End Sub

Private Sub Session_SessionClosed()

    ' Clear the reference
    Set Session = Nothing
    
    ' Close the shell
    Shell.Running = False

End Sub

Private Sub Session_SessionConnected()

    ' Send the logon string
    If LenB(Terminal.LogoffString) <> 0 Then
        Session.SendData Terminal.LogonString & vbCrLf
    End If
    
    ' Save the variables
    Set Commands.Terminal = Terminal
    Set Commands.Session = Session

End Sub

Private Sub Session_SessionExiting()

    ' Send the logoff string
    If LenB(Terminal.LogoffString) <> 0 Then
        Session.SendData Terminal.LogoffString & vbCrLf
    End If

End Sub

Private Sub Shell_ShellExecuting()

    ' Clear variables
    SilienceShell = False

End Sub

Private Sub Shell_ShellInput(Data() As Byte)

    ' Send the data to the client if we're not silencing the shell
    If Not SilienceShell Then
        Session.SendData Data
    End If

End Sub

Private Sub Session_SessionDataRecieved(Data As Variant)

    Dim aData() As Byte, bRun As Boolean

    ' Return whether or not the shell is running
    bRun = Shell.Running

    ' See if there's any point retrieving the ASCII byte array
    If bRun Or EnableCmd Then
    
        ' Convert the string to an ASCII byte array
        aData = StrConv(Data, vbFromUnicode)
    
        ' Parse backspaces and other characters
        ProcessArray aData
        
        ' Now, see if we must parse the data
        If EnableCmd Then
            ParseArray aData
        End If
        
        ' And ... if we're supposed to echo all incomming data, do so
        If EnableEcho Then
            Session.SendData aData
        End If

        ' If the shell is running, ...
        If bRun Then
    
            ' ... write this data to the shell, ...
            Shell.WriteLine aData
        
        Else
        
            ' ... otherwise use the commands.
            ExecuteLogon StrConv(CStr(aData), vbUnicode)
        
        End If
        
    End If

End Sub

Private Function ShiftArray(aData() As Byte, ByVal StartPosition As Long, ByVal Amount As Long) As Boolean

    ' Make sure variables are valid
    If Amount < 1 Then
        Exit Function
    End If

    ' Make sure this is possible
    If Amount + StartPosition <= UBound(aData) Then
        
        ' Do the shift.
        CopyMemory ByVal VarPtr(aData(0)) + StartPosition, ByVal VarPtr(aData(0)) + StartPosition + Amount, _
         UBound(aData) - StartPosition - Amount + 1
        
        ' Clear the last part of the array
        ZeroMemory ByVal (VarPtr(aData(0)) + UBound(aData) - Amount + 1), Amount
        
        ' The operation was successful
        ShiftArray = True
        
    End If

End Function

Public Sub ProcessArray(aData() As Byte)

    Dim Tell As Long, iSize As Long, iStart As Long, bIAC As Boolean, sOutput As String
    
    ' Save the original size
    iStart = LBound(aData)
    iSize = UBound(aData)
    
    ' Go through all the characters
    For Tell = iStart To iSize
    
        ' Do not check for IAC characters if this is the first character
        If Tell > iStart Then
            bIAC = ((aData(Tell - 1) = Telnet_IAC) And (aData(Tell) = Telnet_IAC))
        End If
    
        ' If this is a backspace or IAC character, shift all characters two spaces to the left.
        If aData(Tell) = vbKeyBack Or bIAC Then
        
            ' If the backspace is at the edge
            If (Tell = LBound(aData)) Or bIAC Then
                
                ' Shift only one character insted
                If ShiftArray(aData, Tell - 1, 1) Then
                    Tell = Tell - 1
                End If
                
                ' Decrease the final size nevertheless
                iSize = iSize - 1
                
            Else

                ' Shift this data to the left
                If ShiftArray(aData, Tell - 1, 2) Then
                    Tell = Tell - 2
                End If
                
                ' Decrease the final size by two
                iSize = iSize - 2
            
            End If

        End If
    
    Next

    ' If the new size differes, reallocate the array
    If iSize <> UBound(aData) Then
    
        ' If the size is even smaller than the lower bound, ...
        If iSize < LBound(aData) Then
            ' ... remove the data altoghether.
            Erase aData
        Else
            ' Otherwise, just reallocate.
            ReDim Preserve aData(iSize)
        End If
        
    End If

End Sub

Private Function ProcessCommand(aData() As Byte, iCommand As Byte, iOption As Byte, iNewSize As Long, Tell As Long) As String

    Dim newPos As Long, iFind As Long, bIgnore As Boolean, Pos As Long
    Dim Paramenters() As Byte, bCancel As Boolean, sOutput As String

    ' Clear variable
    newPos = 3

    ' If this is a sub-option negotiation, ...
    If iCommand = Telnet_SB Then
        
        ' ... we'll have to continue onwards.
        For iFind = Tell + 3 To UBound(aData) - 1
        
            ' See if this is the end of the paramenters
            If aData(iFind) = Telnet_IAC And aData(iFind + 1) = Telnet_SE Then
            
                ' We've found the last bit
                Exit For
            
            End If
        
        Next
        
        ' Make sure we found anything
        If iFind > UBound(aData) Then
            bIgnore = True
        Else
        
            ' Make sure there's any parametners
            If iFind - Tell - 4 >= 0 Then
                
                ' Allocate the array to keep the paramenters
                ReDim Paramenters(iFind - Tell - 4)
            
                ' Copy the characters
                CopyMemory Paramenters(0), ByVal (VarPtr(aData(0)) + iFind + Tell + 3), iFind - Tell - 3
                    
                ' Move the search position according to the lenght of the paramenters
                newPos = newPos + GetArrayLenght(Paramenters)
                
            End If
            
            ' Include the two last characters
            newPos = newPos + 2
        
        End If
    
    End If
    
    ' Make sure we aren't ignoring this command
    If Not bIgnore Then
    
        ' Move the search position
        iNewSize = iNewSize - newPos
    
        ' Move all bytes one step down (also, make sure there's anything to copy)
        ShiftArray aData, Tell, newPos
            
         ' Don't move one character to the right
        Tell = Tell - 1
        
        ' Inform that a command has been found
        RaiseEvent CommandExecuting(CLng(iCommand), CLng(iOption), Paramenters, sOutput, bCancel)
        
        ' Add the output to this procedure's output
        ProcessCommand = sOutput
        
        ' Execute this command
        If Not bCancel Then
            ProcessCommand = ProcessCommand & ExecuteCommand(CLng(iCommand), CLng(iOption), Paramenters)
        End If
    
    End If

End Function

Public Sub ParseArray(aData() As Byte)

    Dim Tell As Long, iCommand As Byte, iOption As Byte, iNewSize As Long, sOutput As String
    
    ' Normally, we'll not decrease the resulting size, so use the current size.
    iNewSize = UBound(aData)
    
    ' Go through all the characters
    For Tell = LBound(aData) To UBound(aData)
    
        ' Make sure we're not exceeding the (real) size of the array
        If Tell > iNewSize Then
            Exit For
        End If
    
        ' See if this is the start of a command
        If aData(Tell) = Telnet_IAC And Tell <= UBound(aData) - 2 Then
        
            ' Retrieve the command and the option
            iCommand = aData(Tell + 1)
            iOption = aData(Tell + 2)
            
            ' Continue the process
            sOutput = sOutput & ProcessCommand(aData, iCommand, iOption, iNewSize, Tell)
    
        End If
    
    Next
    
    ' If the command processing generated any output, send it.
    If LenB(sOutput) <> 0 Then
        Session.SendData sOutput
    End If
    
    ' Resize the string, if needed
    If UBound(aData) <> iNewSize Then
        If iNewSize < LBound(aData) Then
            Erase aData
        Else
            ReDim Preserve aData(iNewSize)
        End If
    End If

End Sub

' Converts all given variables to their corresponding value.
' Usage: ConvertVariables yourString, varname1, varvalue1, varname2, varvalue2, ect.
Private Sub ConvertVariables(sText As String, ParamArray Variables() As Variant)

    Dim Tell As Long

    ' Go through all variables
    For Tell = LBound(Variables) To UBound(Variables) Step 2
    
        ' First, make sure the text contains this variable
        If InStr(1, sText, Variables(Tell)) <> 0 Then
            ' Then, convert this variable.
            sText = Replace(sText, Variables(Tell), Variables(Tell + 1))
        End If
    
    Next

End Sub

Public Sub ExecuteLogon(Line As String, Optional TreeDepth As Long)

    On Error Resume Next
    Dim aSegments As Variant, sCommand As String, sParam As String, bCancel As Boolean

    ' When logging on we'll only use simple functions that can be expressed as follows:
    ' [command name] [optional: paramenter]
    
    ' Limit the amount of times this procedure can call itself to (default) 32
    If TreeDepth > MaxTree Then
        SendLine Respond_OutOfStackSpace
        Exit Sub
    End If
    
    ' Make sure there's anything to run
    If LenB(Line) = 0 Then
        Exit Sub
    End If
    
    ' First, we'll split the line into two
    aSegments = Split(Mid(Line, 1, Len(Line) - Len(Session.LineDelimiter)), " ", 2)
    
    ' Retrieve the command and the paramenters
    sCommand = LCase(aSegments(0))
    sParam = aSegments(1)
    
    ' If set, ...
    If EnableEnviroment Then
        ' ... convert enviroment variables
        ConvertVariables sParam, "%username%", Session.CurrentUser.UserName, "%password%", Session.CurrentUser.UserPassword, _
         "%command%", Commands.SavedCommandLine, "%last%", Commands.SavedLine, "%level%", _
          Commands.SavedLevel, "uservar", Commands.SavedUsername, "userpass", Commands.SavedPassword
    End If
    
    ' Inform that the user has sent a command
    RaiseEvent LogonExecuting(sCommand, sParam, bCancel)
    
    ' If we are to cancel this execution, ...
    If bCancel Then
        ' ... do so.
        Exit Sub
    End If

    ' Reference classes
    Set Commands.Session = Session
    Set Commands.Terminal = Terminal

    ' Clear all errors
    Err.Clear

    ' Attempt to call the given command
    CallByName Commands, sCommand, VbMethod, sParam
    
    ' If a error occured, see if it can be any of the following functions insted.
    If Err.Number <> 0 Then
    
        ' Run all other commands as well
        Select Case sCommand
            Case "exec"
        
                ' Just run the code (TreeDepth is a safety measure, to avoid someone figuring out how
                ' to run a code that runs itself.)
                ExecuteLogon Line & vbCrLf, TreeDepth + 1
            
            Case "exit", "close", "quit"
            
                ' Exit the session
                Session.ExitSession

            Case "?", "help"
            
                ' List all commands
                SendLine Respond_SendingHelp, HelpText
        
            Case "rem"
                
                ' Comment. Ignore this.
        
            Case Else
            
                ' Inform that the command wasn't recognized
                SendLine Respond_CommandNotRecognized
        
        End Select
    
    End If

    ' Save this line
    Commands.SavedLine = Line

End Sub

Public Sub SendLine(CodeID As LogonResponses, Optional ExtraData As String, _
 Optional Sender As String, Optional Pretext As String)

    ' The different types of code ids are as follows:
    '  1xx = Negative responses and error messages.
    '  2xx = Positive responses to a command.
    '  3xx = Return data, mostly lists.
    Dim Description As String

    ' Retrieve the description to use
    Select Case CodeID
        Case Respond_AccessDenied: Description = "Access denied."
        Case Respond_UserNotFound: Description = "User not found."
        Case Respond_IncorrectPassword: Description = "Password is incorrect."
        Case Respond_CommandNotRecognized: Description = "Command not recognized."
        Case Respond_ShellFailed: Description = "Shell failed."
        Case Respond_UserAlreadyExist: Description = "User already exist."
        Case Respond_UserExpired: Description = "User account expired."
        Case Respond_OutOfStackSpace: Description = "Out of stack space."
        Case Respond_SessionsExceeded: Description = "The sessions limit has been reached."
        Case Respond_SecurityLevelToHigh: Description = "The user's security level is to high."
        Case Respond_AttributeDosentExist: Description = "This attribute dosen't exits."
        Case Respond_CannotBePerfomedOnSelf: Description = "Cannot modify the current user."
        Case Respond_LogonDenied: Description = "Logon denied."
        Case Respond_ShellAborted: Description = "Shell was aborted."
        Case Respond_CannotContainSpace: Description = " cannot contain space."
        Case Respond_CannotBeNumeric: Description = " cannot contain only numbers."
        Case Respond_ShellFinished: Description = "Shell finished."
        Case Respond_LoggedOn: Description = "Logged on."
        Case Respond_LoggedOff: Description = "Logoff was successful."
        Case Respond_PrivilegiesSet: Description = "Privilegies set."
        Case Respond_Pong: Description = "Pong."
        Case Respond_UserCreated: Description = "User created."
        Case Respond_UserDeleted: Description = "User deleted."
        Case Respond_CommandLineSet: Description = "Command line set."
        Case Respond_LineSent: Description = "Line sent."
        Case Respond_AttributeSet: Description = "The attribute has been set."
        Case Respond_UsersLoggedOn: Description = "Users logged on:"
        Case Respond_AllUsers: Description = "All users:"
        Case Respond_SendingHelp: Description = "Sending help:"
        Case Respond_AttributeGet: Description = "Value:"
    End Select

    ' Send the respond message
    If Not (Session Is Nothing) Then
        Session.SendData CodeID & " " & Pretext & Description & " " & Sender & vbCrLf & ExtraData
    End If

End Sub

Public Function ExecuteCommand(Command As Telnet_Commands, Options As Telnet_Options, Paramenters() As Byte) As String

    On Error Resume Next

    ' Execute the different commands
    Select Case Command
        Case Telnet_WILL, Telnet_DO, Telnet_WONT, Telnet_DONT
            
            ' See what we're supposed to change
            Select Case Options
                Case Telnet_Echo
                    ExecuteCommand = OptionNegotiation(Command, Options, True, EnableEcho)
                Case Telnet_AO ' Abort output
                    ExecuteCommand = OptionNegotiation(Command, Options, True, SilienceShell) ' To accomplish this, we'll just silence the shell.
                Case Telnet_GA ' Go ahead
                    ExecuteCommand = OptionNegotiation(Command, Options, True, SuppressGoAhead)
                Case Telnet_WindowSize ', Telnet_EnvironmentVariables
                    ExecuteCommand = OptionNegotiation(Command, Options, True)
                Case Else
                    ' Inform that this is not supported
                    ExecuteCommand = OptionNegotiation(Command, Options, False)
                End Select
        
        Case Telnet_SB ' Execute functions
            
            ' See what function to execute
            Select Case Options
                Case Telnet_WindowSize
                    
                    ' Set the new window size
                    WindowWidth = (Paramenters(0) * &H100) + Paramenters(1)
                    WindowHeight = (Paramenters(2) * &H100) + Paramenters(3)
                    
                'Case Telnet_EnvironmentVariables
                
            End Select
        
        Case Telnet_AYT
            ' Respond with a NOP-command
            ExecuteCommand = CreateCmd(Telnet_NOP, 0)
        Case Telnet_EL ' Erase line
            Session.DataBuffer = ""
        Case Telnet_IP ' Interrupt process = kill shell
            Shell.Running = False
        Case Else
            
            ' All other functions, if DO or WILL is used, will not be used
            If (Options = Telnet_DO) Or (Options = Telnet_DO) Then
                ExecuteCommand = OptionNegotiation(Command, Options, False)
            End If
            
    End Select

End Function

Private Function OptionNegotiation(Command As Telnet_Commands, Options As Telnet_Options, Support As Boolean, Optional Value As Boolean) As String
    
    ' Enable as default
    Value = IIf(Support, True, False)

    Select Case Command
        Case Telnet_WILL
            ' Enable option and send confirmation
            OptionNegotiation = CreateCmd(IIf(Support, Telnet_DO, Telnet_DONT), Options)
        Case Telnet_DO
            OptionNegotiation = CreateCmd(IIf(Support, Telnet_WILL, Telnet_WONT), Options)
        Case Telnet_WONT
            ' Disable option
            Value = False
            OptionNegotiation = CreateCmd(Telnet_DONT, Options)
        Case Telnet_DONT
            ' As the above
            Value = False
            OptionNegotiation = CreateCmd(Telnet_WONT, Options)
    End Select

End Function

' Used to create a telnet command string
Public Function CreateCmd(Command As Telnet_Commands, Options As Telnet_Options, Optional Paramenters As Variant) As String
    
    ' In all commands there is three bytes divided into IAC, Command and Option
    CreateCmd = Chr(Telnet_IAC) & Chr(Command) & Chr(Options)
    
     ' But, the following command require paramenters:
    If Command = Telnet_SB Then
         ' Using sub-option negotiation, we must include values and thus support more than 3 bytes.
        CreateCmd = CreateCmd & ConvertParamenters(Paramenters) & Chr(Telnet_IAC) & Chr(Telnet_SE)
    End If

End Function

Private Function ConvertParamenters(Elements As Variant) As String
    
    Dim Tell As Long
    
    ' See if the given variable actually is an array
    If Not IsArray(Elements) Then
        Exit Function
    End If
    
    ' Allocate string
    ConvertParamenters = Space(UBound(Elements) - LBound(Elements) + 1)
    
    ' Convert the given array to a string
    For Tell = LBound(Elements) To UBound(Elements)
        ' Set each character
        Mid(ConvertParamenters, Tell - LBound(Elements) + 1, 1) = Chr(Elements(Tell))
    Next
    
End Function


