VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsTerminal"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Different terminal events
Public Event ConnectionRequest(requestID As Long, Cancel As Boolean)
Public Event ConnectionAborted(requestID As Long, Reason As String)
Public Event SessionCreated(Session As clsSession)
Public Event SessionRemoving(Session As clsSession)
Public Event SessionAttemptingLogon(Session As clsSession, RequestedUser As clsUser, Cancel As Boolean)
Public Event SessionExecuting(CommandLine As Variant, AppPath As Variant, Cancel As Boolean)
Public Event UserRequestedShutdown(Caller As clsUser)
Public Event UserCreating(User As clsUser, Cancel As Boolean)
Public Event UserDeleting(User As clsUser, Cancel As Boolean)
Public Event UserListChanged()
Public Event SettingsLoaded()
Public Event SettingsSaved()
Public Event ServerStarted()
Public Event ServerStopped()

' Initialize different users
Public Users As New Collection

' Contains all sessions
Public Sessions As New Collection

' Whether or not to enable logging
Public UseLogging As Boolean
Public LogPath As String ' The path to any log
Public SettingsPath As String ' The path to any setting file

' The time the server was started
Public ServerStarted As Date

' Messages to send automatically when a session connects and disconnects
Public LogonString As String
Public LogoffString As String

' Contains the amount of data send and recieved
Public BytesSent As Double
Public BytesRecieved As Double

' Whether or not the server is currently running
Public Running As Boolean

' Our main server socket
Public WithEvents Socket As clsSocket
Attribute Socket.VB_VarHelpID = -1

' The maximum amount of connections/sessions
Public MaxConnections As Long

' Used to retrieve/store data in XML-files
Public XML As New clsXML

Public Function InvokeEvent(EventID As Long, ParamArray Paramenters() As Variant) As Boolean
    
    Dim oSession As clsSession, oUser As clsUser
    
    ' Used to retrieve variables
    If (EventID = 1) Then
        ' Retrieve variables
        Set oSession = Paramenters(0)
        Set oUser = Paramenters(1)
    End If
    
    ' Invoke different events
    Select Case EventID
        Case 1
        
            ' Let the parent respond to this attempt
            RaiseEvent SessionAttemptingLogon(oSession, oUser, InvokeEvent)
            
        Case 2
        
            ' Inform that we're about to execute an application
            RaiseEvent SessionExecuting(Paramenters(0), Paramenters(1), InvokeEvent)

    End Select

End Function

Public Property Get DefaultUser() As clsUser

    On Error Resume Next
    
    ' Return the default user
    Set DefaultUser = Users("Default")

End Property

Public Function KillSession(Session As clsSession)

    Dim Tell As Long

    ' Firstly, find the session in the list
    For Tell = 1 To Sessions.Count
    
        ' See if these corresponds
        If Session Is Sessions(Tell) Then
        
            ' Close this session
            UnloadSession Session
            
            ' Then, remove this session
            Sessions.Remove Tell
        
        End If
    
    Next

End Function

Private Sub UnloadSession(Session As clsSession)

    On Error Resume Next

    ' Close its shell.
    Session.EventTrapper.Shell.Running = False

    ' Log off the user
    Session.LogOff
    
    ' Add to the amount of bytes sent and recieved
    BytesSent = BytesSent + Session.BytesSent
    BytesRecieved = BytesRecieved + Session.BytesRecieved
    
    ' Clear statistics
    Session.BytesSent = 0
    Session.BytesRecieved = 0
    
    ' Inform that it is about to get removed
    RaiseEvent SessionRemoving(Session)

End Sub

Public Function CreateUser(UserName As String, Password As String) As clsUser

    Dim bCancel As Boolean

    ' Create a new instance
    Set CreateUser = New clsUser
    
    ' Set the properties
    With CreateUser
        .UserName = UserName
        .UserPassword = Password
        .Expire = DateSerial(2099, 1, 1)
        Set .Parent = Me
    End With

    ' Inform about this
    RaiseEvent UserCreating(CreateUser, bCancel)

    ' See if the creation was canceled
    If Not bCancel Then
        
        ' Add the user to the list
        Users.Add CreateUser, UserName
    
        ' The amount of users has changed
        RaiseEvent UserListChanged
    
    End If

End Function

Public Function CreateSession(ByVal requestID As Long) As clsSession

    ' Firstly, create a new instance
    Set CreateSession = New clsSession
    
    ' Use the object
    With CreateSession
    
        ' Accept this connection
        .Socket.Accept requestID
        
        ' Save the remote host
        .RemoteHost = .Socket.RemoteHost
        
        ' Finally, initialize it (use the amount of seconds since 1970 to store the time)
        .InitializeSession IIf(UseLogging, LogPath & .RemoteHost _
         & " - " & DateDiff("s", DateSerial(1970, 1, 1), Now) & " - " & Sessions.Count + 1 & " .txt", ""), Me, New clsTelnet
        
        ' Append the session to the list
        Sessions.Add CreateSession
    
        ' Then set the default user
        .Logon DefaultUser
        
        ' Inform about this creation
        RaiseEvent SessionCreated(CreateSession)
    
    End With

End Function

Public Sub ClearAll()

    Dim Session As clsSession
    
    ' Go through all sessions
    For Each Session In Sessions
                
        ' Log off
        UnloadSession Session
        
    Next

    ' Remove all sessions
    Set Sessions = New Collection
    
    ' Close connection
    Set Socket = Nothing
    
    ' Disable the running flag
    Running = False
    
    ' Inform that the server has (in effect) been stopped
    RaiseEvent ServerStopped
    
End Sub

Public Function KillUser(UserName As String) As Boolean

    Dim iIndex As Long, bCancel As Boolean

    ' Firstly, find the index to the user
    iIndex = FindUser(UserName, , True)
    
    ' If this user is not in the list, do not comply
    If iIndex > 0 Then
    
        ' Inform about this
        RaiseEvent UserDeleting(Users(iIndex), bCancel)
 
        ' Whether or not to delete this user
        If Not bCancel Then
 
            ' Kill this user
            Users.Remove iIndex
        
            ' The list of users has changed
            RaiseEvent UserListChanged
        
            ' Success!
            KillUser = True
    
        End If
    
    End If

End Function

Public Sub RequestShutdown(User As clsUser)

    ' Inform that a user has requested the terminal to shut down
    RaiseEvent UserRequestedShutdown(User)

End Sub

Public Sub Listen(ByVal lngPort As Long)

    ' Clear all sessions
    ClearAll

    ' Create a new socket
    Set Socket = New clsSocket

    ' Start listening for connection requests
    Socket.LocalPort = lngPort
    Socket.Listen
    
    ' Inform that we're runnng
    Running = True
    
    ' Save this date
    ServerStarted = Now

    ' Inform that the server has been started
    RaiseEvent ServerStarted

End Sub

Public Sub BroadcastData(vMessage As Variant, Optional Exception As clsSession)

    Dim oSession As clsSession
    
    ' Go through all sessions, ...
    For Each oSession In Sessions
        
        ' ... except a certain session, ...
        If Not (oSession Is Exception) Then
        
            ' ... sending the same message
            oSession.SendData vMessage
    
        End If
    
    Next

End Sub

Public Function FindUser(Value As Variant, Optional sProperty As String = "UserName", Optional ReturnIndex As Boolean) As Variant

    Dim oUser As clsUser, Index As Long
    
    ' Run through the collection of users, ...
    For Each oUser In Users
    
        ' Increase index
        Index = Index + 1
    
        ' ... returning the user if the string corresponds to the property
        If CallByName(oUser, sProperty, VbGet) = Value Then
            
            ' Return the user if said to
            If ReturnIndex Then
                FindUser = Index
            Else
                Set FindUser = oUser
            End If
            
            ' We're done - exit prosedure
            Exit Function
            
        End If
    
    Next
    
    ' If we're not supposed to return a index, ...
    If Not ReturnIndex Then
        ' ... return an empty object.
        Set FindUser = Nothing
    End If

End Function

Public Sub LoadSettings()

    Dim oTemp As Collection

    ' Retrieve the list of users
    Set oTemp = XML.LoadFile(SettingsPath & "Users.xml")

    ' Only set the list of its not empty
    If Not (oTemp Is Nothing) Then
        Set Users = oTemp
    End If
    
    ' Inform that we're ready to load settings
    RaiseEvent SettingsLoaded
    
End Sub

Public Sub SaveSettings()

    ' Save all collections and objects
    XML.SaveFile Users, SettingsPath & "Users.xml", "users"
    
    ' Inform that we're ready to save settings
    RaiseEvent SettingsSaved

End Sub

Private Sub Class_Initialize()

    ' Initialize the creatable classes
    XML.Classes.Add New clsUser, "clsuser"
    XML.Classes.Add New clsSettings, "clssettings"

    ' Set defaults
    MaxConnections = 10

    ' Load the default user
    With CreateUser("Default", "")
        .AllowLogon = True  ' Allow any user to log on
    End With

End Sub

Private Sub Class_Terminate()

    ' Automatically save everything
    SaveSettings
    
    ' Clear everything
    ClearAll

End Sub

Private Sub Socket_OnConnectionRequest(ByVal requestID As Long)

    Dim bCancel As Boolean

    ' Make sure we haven't exceeded the maximum amount of sessions
    If Sessions.Count <= MaxConnections Then

        ' Inform about this request
        RaiseEvent ConnectionRequest(requestID, bCancel)
    
        ' Add the session as default
        If Not bCancel Then
            CreateSession requestID
        End If
        
    Else
        
        ' Inform that the connection was automatically aborted
        RaiseEvent ConnectionAborted(requestID, "Maximum amount of connections reached.")
    
    End If

End Sub



