VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsXML"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Used to save and retrieve collection of objects in an XML-document
' IMPORTANT: NEVER USE SAVE_ATTRIBUTE AND SAVE_TEXT TO SAVE AN OBJECT OR A DATE.

' The different ways a property can be saved (or not saved at all)
Public Enum SaveTypes
    Save_Element = 0
    Save_Attribute = 1
    Save_Text = 2
    Save_Disallow = 3
End Enum

' Used to retrieve the name of a object
Private TLI As New TLIApplication

' Collection that contains all the classes that is savable and loadable
Public Classes As New Collection

Public Function SaveFile(refCollection As Collection, sFile As String, Name As String) As Boolean

    Dim oDocument As New DOMDocument, oParent As IXMLDOMElement

    ' Create a parent-element
    Set oParent = oDocument.CreateElement("xml")

    ' Append this element to the document
    oDocument.appendChild oParent
    
    ' Append the given collection
    AppendCollection refCollection, Name, oParent
    
    ' Format the resulting XML
    FormatXML oParent, 3
    
    ' Finally, save the resulting document
    oDocument.save sFile

    ' Saving was successful
    SaveFile = True

End Function

Public Function LoadFile(sFile As String) As Collection

    Dim oDocument As New DOMDocument
    
    ' Load the given file
    If oDocument.Load(sFile) Then
    
        ' Load the collection saved within this file (<XML> -> <(name)s>)
        Set LoadFile = LoadCollection(oDocument.firstChild.firstChild)

    End If

End Function

Public Function LoadCollection(parentNode As IXMLDOMElement) As Collection

    On Local Error Resume Next
    Dim oCollection As New Collection, oElement As IXMLDOMElement, oObject As Object, sDefault As String

    ' Go through all elements
    For Each oElement In parentNode.childNodes
    
        ' Reset variables
        sDefault = ""
    
        ' Load the current element
        Set oObject = LoadObject(oElement)

        ' Retrieve the default value
        sDefault = CStr(oObject)

        ' After which we'll add it to the collection
        If LenB(sDefault) = 0 Then
            oCollection.Add oObject
        Else
            oCollection.Add oObject, sDefault
        End If

    Next
    
    ' Return the collection
    Set LoadCollection = oCollection

End Function

Public Function LoadObject(Node As IXMLDOMElement) As Object

    On Local Error Resume Next
    Dim Member As MemberInfo, vValue As Variant, bSet As Boolean

    ' Clone the object to use
    Set LoadObject = Classes("cls" & LCase(Node.getAttribute("type"))).Clone

    ' Go trough all properties
    For Each Member In TLI.InterfaceInfoFromObject(LoadObject).Members
    
        ' Clear value
        vValue = ""
    
        ' See if this is a property with just one paramenter (that is, the input)
        If Member.Parameters.Count = 0 And Member.InvokeKind = INVOKE_PROPERTYPUT Then
        
            ' Assume we'll set this property as default
            bSet = True
        
            ' Retrieve how we'll act further
            Select Case LoadObject.XML(Member.Name)
                Case Save_Element
                
                    ' Load the element
                    SafeSet vValue, LoadAsElement(Node, Member)
                    
                Case Save_Attribute
                
                    ' Retrieve using a attribute
                    vValue = Node.getAttribute(LCase(Member.Name))
                
                Case Save_Text
                
                    ' Use the text-element
                    vValue = Node.Text
                
                Case Save_Disallow
                    
                    ' Don't allow this element
                    bSet = False
            
            End Select
            
            ' Just set this value
            If bSet Then
                CallByName LoadObject, Member.Name, VbLet, vValue
            End If
            
        End If
            
    Next

End Function

Public Sub SaveObject(parentNode As IXMLDOMElement, oElement As Object, Name As String)
    
    Dim oNode As IXMLDOMElement, Member As MemberInfo
    Dim oObject As Object, retSave As Long, vValue, sNewName As String
    
    ' DO not save an empty object
    If oElement Is Nothing Then
        Exit Sub
    End If
    
    ' Create a new element within the container
    Set oNode = CreateElement(parentNode, Name)

    ' Set the type of the element (remove the first three characters)
    oNode.setAttribute "type", LCase(Mid(GetObjectName(oElement), 4))

    ' Go trough all properties
    For Each Member In TLI.InterfaceInfoFromObject(oElement).Members

        ' See if this is a property with just one paramenter (that is, the input)
        If Member.Parameters.Count = 0 And Member.InvokeKind = INVOKE_PROPERTYGET And Member.Name <> "_Default" Then

            ' Get how to save this element
            retSave = oElement.XML(Member.Name)

            ' See if we are supposed to save this element
            If retSave <> Save_Disallow Then
            
                ' Get the value
                SafeSet vValue, CallByName(oElement, Member.Name, VbGet)
    
                ' Set the name of the new member
                sNewName = LCase(Member.Name)
    
                ' Retrieve how we'll act further
                Select Case retSave
                    Case Save_Element
                    
                        ' Create a new element with this information
                        SaveAsElement oNode, oElement, vValue, sNewName
                        
                    Case Save_Attribute
                    
                        ' Just set an attribute
                        oNode.setAttribute sNewName, ConvertToString(vValue)
                    
                    Case Save_Text
                    
                        ' Use the text-element
                        oNode.Text = ConvertToString(vValue)
                
                End Select
            
            End If

        End If

    Next

End Sub

Private Function LoadAsElement(Node As IXMLDOMElement, Member As MemberInfo) As Variant

    On Local Error Resume Next
    Dim oSelected As IXMLDOMElement, sType As String

    ' Retrieve subelement
    Set oSelected = Node.selectSingleNode(LCase(Member.Name))
    
    ' Try to retrieve the type variable
    sType = LCase(oSelected.getAttribute("type"))
    
    ' See if this element contains subelements (when it have a type-attribute), ...
    If sType <> "" Then
    
        ' ... load it as a object.
        If sType = "collection" Then
            Set LoadAsElement = LoadCollection(oSelected)
        ElseIf sType = "date" Then
            ' If it is a date, convert and return.
            With oSelected
                LoadAsElement = DateSerial(.getAttribute("year"), .getAttribute("month"), .getAttribute("day")) + _
                          TimeSerial(.getAttribute("hour"), .getAttribute("minute"), .getAttribute("second"))
            End With
        Else
            Set LoadAsElement = LoadObject(oSelected)
        End If
        
    Else
    
        ' Just return the content
        LoadAsElement = oSelected.Text
    
    End If

End Function

Private Sub SaveAsElement(Parent As IXMLDOMElement, oElement As Object, vValue As Variant, Name As String)

    Dim oCollection As Collection, oObject As Object

    ' Firstly, see if this is an object.
    If IsObject(vValue) Then
        
        ' If so, and it is a collection ...
        If TypeOf vValue Is Collection Then
            ' ... save the object as a collection.
            Set oCollection = vValue
            AppendCollection oCollection, Name, oElement
        Else
            ' Otherwise, save as an object.
            Set oObject = vValue
            SaveObject oElement, oObject, Name
        End If
        
    Else
        
        ' Check the type of this variable
        If varType(vValue) = vbDate Then
            
            ' If this is a date, create a new element, ...
            With CreateElement(Parent, Name)
                ' ... and save the value of the data as attributes.
                .setAttribute "year", Year(vValue)
                .setAttribute "month", Month(vValue)
                .setAttribute "day", Day(vValue)
                .setAttribute "hour", Hour(vValue)
                .setAttribute "minute", Minute(vValue)
                .setAttribute "second", Second(vValue)
                .setAttribute "type", "date"
            End With
            
        Else
        
            ' Convert this to a string.
            CreateElement Parent, Name, ConvertToString(vValue)
            
        End If
        
    End If

End Sub

Private Function ConvertToString(Value As Variant) As String

    ' See then what type this value is.
    If varType(Value) = vbBoolean Then
        
        ' Boolean values should be saved as 1 and 0.
        ConvertToString = IIf(Value, 1, 0)
        
    Else
    
        ' Otherwisem, we'll just let VB take care of the conversion.
        ConvertToString = Value
    
    End If
        
End Function

Public Sub AppendCollection(refCollection As Collection, Name As String, parentNode As IXMLDOMElement)

    Dim oElement As Object, oContainer As IXMLDOMElement, sName As String
    
    ' Create the container
    Set oContainer = CreateElement(parentNode, Name)
    
    ' Set the node type
    oContainer.setAttribute "type", "collection"
    
    ' Go through all its elements
    For Each oElement In refCollection
        
        ' Make sure the element exist
        If Not (oElement Is Nothing) Then
        
            ' No need to get the name twice
            If LenB(sName) = 0 Then
                sName = LCase(Mid(GetObjectName(oElement), 4))
            End If
            
            ' Save each and every one of them
            SaveObject oContainer, oElement, sName
        
        End If
        
    Next

End Sub

Public Function CreateElement(refParent As IXMLDOMElement, Name As String, Optional Text As String) As IXMLDOMElement

    ' Firstly, create the element
    Set CreateElement = refParent.ownerDocument.CreateElement(Name)
    
    ' Set its content
    If LenB(Text) <> 0 Then
        CreateElement.Text = Text
    End If
    
    ' Append the element to the parent
    refParent.appendChild CreateElement

End Function

Private Sub FormatXML(parentNode As IXMLDOMElement, ByVal iIndent As Integer)

    Dim Child As IXMLDOMNode, Document As DOMDocument, bLast As Boolean
    
    ' Retrieve the current document
    Set Document = parentNode.ownerDocument

    ' Go through all the childs in this node
    For Each Child In parentNode.childNodes
    
        ' Do not process this for other node types
        If Child.nodeType = NODE_ELEMENT Then
    
            ' Whether or not this is the last child element
            bLast = CBool(Child Is parentNode.lastChild)
        
            ' Add the beginning seperators for this node
            parentNode.insertBefore Document.createTextNode(vbCrLf & Space(iIndent)), Child
            
            ' And then the ending seperator if we're at the end
            If bLast Then
                parentNode.appendChild Document.createTextNode(vbCrLf & Space(iIndent - 3))
            End If
    
            ' Search for seperators to add in this child's nodes
            FormatXML Child, iIndent + 3
            
        End If
    
    Next

End Sub

Public Function GetObjectName(refObject As Object) As String
    
    ' Load the information from the object
    With TLI.InterfaceInfoFromObject(refObject)
        
        ' Return the class' name (without underscors)
        GetObjectName = Mid(.Name, 2)

    End With

End Function

Public Function GetProcedureList(refObject As Object) As Object
    
    ' Load the information from the object
    With TLI.InterfaceInfoFromObject(refObject)
        
        ' Return the list of procedures
        Set GetProcedureList = .Members

    End With

End Function

Private Sub SafeSet(vDestination, vSource)

    ' Now, if the source is a object, ...
    If IsObject(vSource) Then
        ' ... set the destination the correct way
        Set vDestination = vSource
    Else
        ' If not, just set the variable
        vDestination = vSource
    End If

End Sub

